@@ -1,4 +1,5 @@
 import random
+import time
 
 from typing import List, Union
 from enum import Enum
@@ -139,7 +140,7 @@ CSS = """
     border-radius: 6px;
 }
 
-.nbae-quiz .final {
+.nbae-quiz .main-score {
     font-weight: bold;
     font-size: larger;
     padding-left: 6px;
@@ -235,6 +236,10 @@ CSS = """
 }
 """
 
+def now():
+    return dict(date=time.strftime("%Y-%m-%d"),
+                time=time.strftime("%H:%M:%S"))
+
 # unless specified otherwise in yaml
 DEFAULT_CONTENT_CLASS = MarkdownMathContent
 
@@ -613,9 +618,15 @@ class QuizQuestion:
         self.checkboxes = [Checkbox(value=option.selected, disabled=False,
                                     description='', indent=False)
                            for option in self._displayed_options]
-        # radio-box behaviour
-        if self.exactly_one_option:
-            for checkbox in self.checkboxes:
+        # arm callback so that choices get saved at all times
+        # xxx not working at this point as we have no means
+        # to retrieve the parent Quiz object to trigger a save_preserved
+        #def preserve(b):
+        #    self.xxx_quiz_xxx.save_preserved()
+        for checkbox in self.checkboxes:
+            # checkbox.observe(preserve)
+            # radio-box behaviour
+            if self.exactly_one_option:
                 checkbox.observe(lambda event: self.radio_button_callback(event))
         labels = [option.render().widget() for option in self._displayed_options]
         
@@ -722,6 +733,7 @@ class QuizQuestion:
         for option, checkbox in zip(self._displayed_options, self.checkboxes):
             option.selected = checkbox.value
         return [option.selected for option in self.options]
+    
             
     def restore(self, bools: List[bool]):
         for option, boolean in zip(self.options, bools):
@@ -773,7 +785,7 @@ class Quiz:
         self.displayed_questions = _DisplayedQuestions(self.questions, self.shuffle)
         # needs to be saved somewhere
         self.current_attempts = storage_read(self.exoname, 'current_attempts', 0)
-        preserved = storage_read(self.exoname, "answers", [])
+        preserved = storage_read(self.exoname, "preserved", [])
         if preserved: 
             self.restore(preserved)
         # set question rank
@@ -792,7 +804,7 @@ class Quiz:
         sons.append(HBox([self.submit_button, self.submit_summary])
                     .add_class('result-area'))
         toplevel = VBox(sons).add_class('nbae-quiz')
-        self.update()
+        self.update(within_submit=False)
         return toplevel
 
     
@@ -802,9 +814,11 @@ class Quiz:
         #  we need to disable this until the event is processed
         # update() will re-enable it later on if needed
         self.submit_button.disabled = True
-        self.update()
+        self.update(within_submit=True)
         storage_save(self.exoname, 'current_attempts', self.current_attempts)
-        storage_save(self.exoname, "answers", self.preserve())
+        self.save_submitted()
+        # xxx no longer needed if we can get all changes to be saved
+        self.save_preserved()
         (current_score, max_score,
          normalized_score, normalized_max_score) = self.total_score()
         log_kwds = {}
@@ -825,36 +839,67 @@ class Quiz:
             question.restore(list_of_bools)
 
 
-    def score_html(self, final):
+    def save_preserved(self):
+        storage_save(self.exoname, "preserved", self.preserve())
+    def save_submitted(self):
+        history = storage_read(self.exoname, "submitted", [])
+        this_attempt = dict(when=now(), answers=self.preserve(), html=self.summary_html)
+        history.append(this_attempt)
+        storage_save(self.exoname, "submitted", history)
+
+
+    # final no longer used
+    def score_html(self, right_answers):
         s, m, ns, nm = self.total_score()
         score = ""
-        if final:
-            score += (f"final score "
-                      f"(after {self.current_attempts}/{self.max_attempts} attempts) ")
-        else:
-            score += (f"score for attempt #{self.current_attempts} ")
+        score += (f"score for attempt #{self.current_attempts}:&nbsp;&nbsp;&nbsp; ")
         if self.max_grade is None:
-            score += f"<span class='final'>{display(s)}</span>/{m}"
+            score += f"<span class='main-score'>{display(s)}/{m}</span>"
         else:
             score += f"{display(s)}/{m}"
-            score += f" = <span class='final'>{display(ns)}/{nm}</span>"
+            score += f" = <span class='main-score'>{display(ns)}/{nm}</span>"
+        score += f" ({len(right_answers)}/{len(self.answers)} questions OK)"
         return score
         
 
-    def update(self):
+    def update(self, *, within_submit):
         # there may be latency if the host is loaded
         self.answers = [question.detailed_answer() 
                         for question in self.displayed_questions]
         right_answers = [answer for answer in self.answers if answer == Answer.RIGHT]
         all_right = (len(right_answers) == len(self.answers))
+
+        history = storage_read(self.exoname, "submitted", [])
+        present = now()
+        today, right_now = present['date'], present['time']
+        def past_entry(event):
+            when = event['when']
+            if when['date'] == today:
+                timestamp = f"@{when['time']}"
+            else:
+                timestamp = f"on{when['date']} @{when['time']}"
+            return f"{timestamp} {event['html']}"
+        past_summaries = [
+            past_entry(event) for event in history
+        ]
+        def update_summary(current_summary):
+            aggregate = ""
+            aggregate += "<div>"
+            for summary in past_summaries:
+                aggregate += f"{summary}<br>"
+            if within_submit:
+                aggregate += f"@{right_now} {current_summary}"
+            aggregate += "</div>"
+            self.submit_summary.value = aggregate
+        # quiz is over - either way (all good or attempts ran out)
         if all_right or self.current_attempts >= self.max_attempts:
             # materialize all questions
             for question in self.displayed_questions:
                 question.feedback(question.detailed_answer())
                 question.individual_feedback()
             self.submit_button.description = "quiz over"
-            summary = self.score_html(final=True)
-            self.submit_summary.value = summary
+            self.summary_html = f"{self.score_html(right_answers)}"
+            update_summary(self.summary_html)
             if all_right:
                 self.submit_summary.add_class('ok')
             else:
@@ -868,9 +913,8 @@ class Quiz:
             self.submit_button.description = (
                 f"submit ({left}/{self.max_attempts} attempts left)")
             if self.current_attempts >= 1:
-                self.submit_summary.value = (
-                    f"{self.score_html(final=False)}"
-                    f" ({len(right_answers)}/{len(self.answers)} questions OK)")
+                self.summary_html = f"{self.score_html(right_answers)}"
+                update_summary(self.summary_html)
             self.submit_button.disabled = False
 
     def total_score(self):
